from Framework import Controller,StaticController,templatepage
from Framework.PluginManager import DeclarePlugin
from cherrypy import expose, request  
from Tools.SecurityModuleCore import RedirectorToLogin
from Tools.SecurityModuleCore import SecurityToken, RedirectToLocalPage, RedirectAway, RedirectorToLogin
from Tools.SecurityModuleCore import Group, Role, NotAuthenticated, FetchFromArgs
from Tools.SecurityModuleCore import is_authorized, is_authenticated, has_site 
from cherrypy.lib import static
import os.path
import pickle
import urllib
import string
import ConfigParser
from sets import Set

from CondDBCommon import RedirectorToLogin

class GlobalTagCfg(object):
    templatesPath = staticmethod (lambda : os.path.join(__file__.rsplit ("/", 1)[0], "Templates"))
    __configdir=os.path.join(__file__.rsplit ("/", 1)[0],'config')
    __configfile=os.path.join(__configdir,'globalTag.ini')

    def pfnList(self):
        cparser=ConfigParser.ConfigParser()        
        cparser.read(self.__configfile)
        return cparser.get('PFN','pfnList')

class GlobalTag(Controller):    
    """ Page uses globalTagMain template as index
    """

    def __init__(self, context):
        self.context=context
        Controller.__init__(self,context,__file__)
        self.__basepath = __file__.rsplit('/', 1)[0]
        self.context.addService (GlobalTagCfg())
        self.__appurl=context.CondDBCommonCfg().appserverURL()
        #template variable dict
        self.ret_dict={}
        self.ret_dict['error'] = None
        self.data = {}

    @expose
    @is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    def index(self):
        """Initializes main page
        """
        self._getMainData()
        ###adding PFN to tag names
        for cstr, data_tup in self.data.items():
        #over DBs
            for tag in data_tup[0]:
                tag['tagname'] = tag['tagname']+' | '+ tag['pfn'][:23]
        ###
        self.ret_dict['mainData'] = self.data
        self.ret_dict['token'] = SecurityToken()
        return self.templatePage("globalTagIndex", self.ret_dict)

    def _getMainData(self):
        """ Fetches overview data
        """
        url = self.__appurl+'/globaltag/overview'
        dataHandle = urllib.urlopen(url)
        _ret = dataHandle.read()
        unsorted = pickle.loads(_ret)
        for db, tuples in unsorted.items():
            tuples[0].sort(lambda x, y: cmp(x['tagname'], y['tagname']))
        self.data = unsorted

    def _getTreeData(self, db, trname):
        """Fetches tree
        """
        db = db.replace("oracle://", "")
        url = self.__appurl+'/globaltag/getTree?'
        url += 'treename='+trname.lstrip()
        url += '&service='+db.split('/')[0]
        url += '&schema='+db.split('/')[1]
        url += '&viacache=true'
        dataHandle = urllib.urlopen(url)
        _ret = dataHandle.read()
        return pickle.loads(_ret)

    def _cloneTree(self, sourceTrName, trgTrName,sourceDB, pfn=None):
        """ in sourcDB clones tree named sourceTrName creating a tree named  trgTrName
            sets pfn if given
        """
        sourceDB = sourceDB.replace("oracle://", "")
        url = self.__appurl+'/globaltag/cloneTree?'
        url += 'sourcetreename='+sourceTrName.lstrip()
        url += '&targettreename='+trgTrName.lstrip()
        url += '&service='+sourceDB.split('/')[0]
        url += '&schema='+sourceDB.split('/')[1]
        if pfn != None:
            url += '&pfnreplace='+pfn
        dataHandle = urllib.urlopen(url)
        _ret = dataHandle.read()
        return pickle.loads(_ret)

    def _copyTree(self, sourceTrName, trgTrName,sourceDB, trgDB, pfn=None):
        """NOT IMPLEMENTED IN APP SERVER
        copys a tree named sourceTrName from sourceDB and saves it
        as a tree named  trgTrName in trgDB
        sets pfn if given
        """
        sourceDB = sourceDB.replace("oracle://", "")
        trgDB = trgDB.replace("oracle://", "")
        url = self.__appurl+'/globaltag/copyTree?'
        url += 'sourcetreename='+sourceTrName.lstrip()
        url += '&targettreename='+trgTrName.lstrip()
        url += '&sourceservice='+sourceDB.split('/')[0]
        url += '&sourceschema='+sourceDB.split('/')[1]
        url += '&targetservice='+trgDB.split('/')[0]
        url += '&targetschema='+trgDB.split('/')[1]
        if pfn != None:
            url += '&pfnreplace='+pfn 
        dataHandle = urllib.urlopen(url)
        _ret = dataHandle.read()
        return pickle.loads(_ret)    

    def _renameNode(self, treename,db, oldName, newName, tagid=None):
        """Renames node in a given tree
        NOT IMPLEMENTED IN APP SERV: possibiliti to assign a tag to tree
        """
        map ="'"+oldName+"':'"+newName+"'"
        db = db.replace("oracle://", "")
        url = self.__appurl+'/globaltag/renameNodes?'
        url += 'treename='+treename.lstrip()
        url += '&service='+db.split('/')[0]
        url += '&schema='+db.split('/')[1]
        url += '&nodenamemap='+map
        urllib.urlopen(url)
        return 
        
    def _insertNode(self,treename,db ,nodeName,parentname, tagId):
        """ Inserts a node to a given tree as a child of a given parent node
        associates given tag with this new node if it's a leaf
        """
       ##  if tagId!=None:
##             #nodeDict = pickle.dumps( {'nodelabel':nodeName, 'tagid':tagId}) # 'globalsince':None, 'globaltill':None,
##             str( {'nodelabel':nodeName, 'tagid':tagId})
##         else:    
##             #nodeDict = pickle.dumps( {'nodelabel':nodeName})# 'globalsince':None, 'globaltill':None
##             str( {'nodelabel':nodeName})
        db = db.replace("oracle://", "")
        url = self.__appurl+'/globaltag/insertNode?'
        url += 'treename='+treename.lstrip()
        url += '&service='+db.split('/')[0]
        url += '&schema='+db.split('/')[1]
        url += '&parentname='+parentname
        url += '&nodelabel='+ nodeName
        url += '&tagid='+ tagId
        dataHandle = urllib.urlopen(url)
        return 

    def _deleteNode(self,treename,db ,nodeName):
        """ deletes a node from a given tree
        """
        db = db.replace("oracle://", "")
        url = self.__appurl+'/globaltag/deleteNode?'
        url += 'treename='+treename.lstrip()
        url += '&nodename='+ nodeName
        url += '&service='+db.split('/')[0]
        url += '&schema='+db.split('/')[1]
        dataHandle = urllib.urlopen(url)
        return

    def _deleteBranch(self,treename,db ,nodeName):
        """deletes abranch from a given tree
        based on the name of branche's root (nodeName)
        """
        db = db.replace("oracle://", "")
        url = self.__appurl+'/globaltag/deleteBranch?'
        url += 'treename='+treename.lstrip()
        url += '&nodename='+ nodeName
        url += '&service='+db.split('/')[0]
        url += '&schema='+db.split('/')[1]
        dataHandle = urllib.urlopen(url)
        return 

    @expose
    @is_authorized(Role("GlobalTag Manager") , Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    def addTag(self):
        """NOT IMPLEMENTED (ALSO IN APP SERVER)
        dummy method to add a tag
        """
        self.ret_dict['popup'] = "addTag.tmpl"
        self.ret_dict['token'] = SecurityToken()
        return self.templatePage("addTag", self.ret_dict)


    @expose
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))  
    def addTreeFromExisting(self, *args, **kwds):
        """Returns a form allowing tree cloning
        """
        self.ret_dict['token'] = SecurityToken()
        self.ret_dict['goalDB'] = kwds['currentDB']
        self.ret_dict['data'] = self.data
        self.ret_dict['pfnList'] = None
        self.ret_dict['selTree'] = None
        self.ret_dict['pfnList']=self.context.GlobalTagCfg().pfnList().split('|')
        return self.templatePage("addTreeFromExisting", self.ret_dict)

    

    """Dummmy methods prepared to add tree
    based on predefined stucture or from scratch
    NOT IMPLEMENTED
    """
  ##   @expose
##     @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
##     #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
##     def addTreeFromScratch(self, *args, **kwds):
##         self.tree = Tree([{'nodelabel':'All', 'parentlabel':'ROOT', 'tagid':0}], 'T1')
##         self.ret_dict['goalDB'] = kwds['currentDB']
##         self.ret_dict['node_list'] = self.tree.serialized
##         self.ret_dict['token'] = SecurityToken()
##         return self.templatePage("addTreeCommon", self.ret_dict)
    
##     @expose
##     @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
##     #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
##     def addTreeFromDummy(self, *args, **kwds):
##         self.ret_dict['goalDB'] = kwds['currentDB']
##         tree = Tree([{'nodelabel':'All', 'parentlabel':'ROOT', 'tagid':0},{'nodelabel':'Calibration', 'parentlabel':'All', 'tagid':0}, \
##                      {'nodelabel':'Alignment', 'parentlabel':'All', 'tagid':0},{'nodelabel':'Test1', 'parentlabel':'Calibration', 'tagid':3}], 'T1')
##         self.ret_dict['node_list'] = self.tree.serialized
##         self.ret_dict['token'] = SecurityToken()
##         return self.templatePage("addTreeCommon", self.ret_dict)

    @expose
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    def cloneTree(self, *args, **kwds):
        """Shows window showing status of cloning after submit
        """
        if (kwds['pfn']==""):
            pfn = None
        else:
            pfn = kwds['pfn']    
        self._cloneTree(kwds['sourceTreeName'], kwds['newTreeName'],kwds['goalDB'], pfn)
        self.ret_dict['cloneFailed'] = False
        return self.templatePage("submitStatus", self.ret_dict)

    @expose
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    def copyTree(self, *args, **kwds):
        """Shows window showing status of copying a tree after submit
        """
        if (kwds['pfn']==""):
            pfn = None
        else:
            pfn = kwds['pfn']
        self._copyTree(kwds['sourceTreeName'], kwds['newTreeName'],kwds['sourceDB'], kwds['goalDB'], kwds['pfn'] )
        self.ret_dict['cloneFailed'] = False
        return self.templatePage("submitStatus", self.ret_dict)
    
    @expose
    #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    def editTree(self, *args, **kwds):
        """ shows window allowing tree edition
        """
        self.ret_dict['error'] = None
        self.ret_dict['changeList']=pickle.dumps([]).replace('\n', 'nL1')
        self.ret_dict['currDB']= kwds['currentDB']
        tree = Tree(self._getTreeData(kwds['currentDB'], kwds['treeName']), kwds['treeName'])
        self.ret_dict['node_list'] = tree.serialized
        self.ret_dict['currDB'] = kwds['currentDB']
        self.ret_dict['treeName'] = kwds['treeName']
        if kwds.has_key('goalDB'):
            self.ret_dict['goalDB'] = kwds['goalDB']
        else:
            self.ret_dict['goalDB'] = False
        self.ret_dict['token'] = SecurityToken()
        return self.templatePage("treeEdit", self.ret_dict)

    @expose
    @is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    def viewTree(self, *args, **kwds):
        """opens tree view window
        """
        self.ret_dict['currDB']= kwds['currentDB']
        tree = Tree(self._getTreeData(kwds['currentDB'], kwds['treeName']), kwds['treeName'])
        self.ret_dict['node_list']=tree.serialized
        self.ret_dict['currDB'] = kwds['currentDB']
        self.ret_dict['treeName'] = kwds['treeName']
        self.ret_dict['token'] = SecurityToken()
        return self.templatePage("treeView", self.ret_dict)

    @expose
    #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    def finalChangeSubmit(self, *args, **kwds):
        """ shows a list of changes that will be applied to a tree
        """
        changeIds = kwds['changeIds'].split('|')
        changeList = pickle.loads(kwds['changeData'].replace('nL1', '\n'))
        while(len(changeList) !=0):
            elem = changeList.pop(0)
            if (str(elem['id']) in changeIds):
                if (elem['operation']=='ADD' and elem['isLeaf']=='False'):
                    self._insertNode(kwds['treeName'],kwds['currentDB'] ,elem['nodeName'],elem['parentNode'], None)
                elif (elem['operation']=='ADD'):
                    self._insertNode(kwds['treeName'],kwds['currentDB'] ,elem['nodeName'],elem['parentNode'],elem['tagid'])
                    
                elif (elem['operation']=='CHANGE' and elem.has_key('tagid')):
                    self._renameNode(kwds['treeName'],kwds['currentDB'], elem['nodeName'], elem['newName'], elem['tagid'])

                elif (elem['operation']=='CHANGE'):
                    self._renameNode(kwds['treeName'],kwds['currentDB'], elem['nodeName'], elem['newName'])
                elif (elem['operation']=='REMOVE'):
                    self._deleteNode(kwds['treeName'],kwds['currentDB'] ,elem['nodeName'])
                elif (elem['operation']=='REMOVE_BRANCH'):
                    self._deleteBranch(kwds['treeName'],kwds['currentDB'] ,elem['nodeName'])
                else:
                    pass
            else:
                pass
        self.ret_dict['cloneFailed'] = False
        return self.templatePage("submitStatus", self.ret_dict)
        
    @expose
    #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    def submitMiddleTreeChange(self, *args, **kwds):
        """Submits changes selected from the list created in the finalChangeSubmit method 
        
        """
        _changes =  pickle.loads(kwds['changeList'].replace('nL1', '\n'))
        hasKids = False
        for elem in _changes:
            if elem['operation']=='ADD' and elem['isLeaf']=='False':
                for node in _changes:
                    if node['operation']=='ADD' and node['parentNode']==elem['nodeName']:
                        hasKids = True
                if not hasKids:
                    _changes.remove(elem)
            hasKids = False
        self.ret_dict['changes'] = _changes
        self.ret_dict['currDB'] = kwds['currentDB']
        self.ret_dict['treeName'] = kwds['treeName']
        return self.templatePage("editChangesList", self.ret_dict)


    """Node operations
    """
    @expose
    #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    def showInsertNode(self, *args, **kwds):
        """Opens node insertion form
        """
        self.ret_dict['currentDB'] = kwds['currentDB']
        self.ret_dict['treeName'] = kwds['treeName']
        self.ret_dict['parent_node'] =  kwds['parent_node']
        self.ret_dict['token'] = SecurityToken()
        self.ret_dict['changeList'] = kwds['changeList']
        self.ret_dict['data'] = self.data
        self.ret_dict['tagList'] = None
        return self.templatePage("addTreeNode", self.ret_dict)

    def _prepTagList(self,currentDB, treeName,changeList ):
        """Prepares a list of tags that can be associated with a node
        during node insertion or edition
        The list consists of all the tags in the inventory that are not associeted
        with tags
        """
        existingTags = []
        avTags = []
        tree = Tree(self._getTreeData(currentDB, treeName), treeName)
        chList = pickle.loads(changeList.replace('nL1', '\n'))
        tree.applyPreviousChanges(chList)
        for node in tree.serialized:
           if node.has_key('tagid'):
               existingTags.append(node['tagid'])
        for tag in self.data["oracle://"+currentDB][0]:
            if tag.has_key('tagid') and tag['tagid'] not in existingTags:
                avTags.append(tag)
        return avTags
    
    @expose
    #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    def showEditNode(self, *args, **kwds):
        """ Opens node edition form
        """
        self.ret_dict['currentDB'] = kwds['currentDB']
        self.ret_dict['treeName'] = kwds['treeName']
        self.ret_dict['nodeName'] = kwds['nodeName']
        self.ret_dict['token'] = SecurityToken()
        self.ret_dict['changeList'] = kwds['changeList']
        self.ret_dict['data'] = self.data
        self.ret_dict['tagList'] = self._prepTagList(kwds['currentDB'],kwds['treeName'],kwds['changeList'])
        return self.templatePage("editTreeNode", self.ret_dict)

    @expose
    #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    def showTagList(self, *args, **kwds):
        """ Shows a list of available tags prepared in  _prepTagList method in the node insertion form
        """
        self.ret_dict['tagList'] = self._prepTagList(kwds['currentDB'],kwds['treeName'],kwds['changeList'])
        self.ret_dict['name'] = kwds['name']
        self.ret_dict['currentDB'] = kwds['currentDB']
        self.ret_dict['treeName'] = kwds['treeName']
        self.ret_dict['parent_node'] =  kwds['parent_node']
        self.ret_dict['token'] = SecurityToken()
        self.ret_dict['changeList'] = kwds['changeList']
        self.ret_dict['data'] = self.data 
        return self.templatePage("addTreeNode", self.ret_dict)

    @expose
    #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    def insert_node(self, *args, **kwds):
        """Applies data about node to be inserted to a change list
        """
        self.ret_dict['error'] = None
        self.ret_dict['currDB'] = kwds['currentDB']
        self.ret_dict['treeName'] = kwds['treeName']
        tree = Tree(self._getTreeData(kwds['currentDB'], kwds['treeName']), kwds['treeName'])
        changeList = pickle.loads(kwds['changeList'].replace('nL1', '\n'))
        tree.applyPreviousChanges(changeList)
        if kwds.has_key('name') and not tree.hasNode(kwds['name']):
            if kwds.has_key('parent_node') and kwds['isLeaf']=='true':
                tree.insert_node(kwds['name'], kwds['parent_node'], kwds['tagId'])
                changeList.append({'operation':'ADD', 'id':len(changeList), 'nodeName':kwds['name'], 'parentNode':kwds['parent_node'], 'isLeaf':'True', 'tagid':kwds['tagId']})
            elif kwds.has_key('parent_node') and kwds['isLeaf']=='false':
                tree.insert_node(kwds['name'], kwds['parent_node'], 0)
                changeList.append({'operation':'ADD', 'id':len(changeList), 'nodeName':kwds['name'], 'parentNode':kwds['parent_node'], 'isLeaf' : 'False',  'tagid':0})
            elif kwds['isLeaf']=='true':
                tree.insert_node(kwds['name'], 'ROOT', kwds['tagId'])
                changeList.append({'operation':'ADD', 'id':len(changeList), 'nodeName':kwds['name'], 'parentNode':'ROOT', 'isLeaf':'True', 'tagid':kwds['tagId']})
            else:
                tree.insert_node(kwds['name'], 'ROOT', 0)
                changeList.append({'operation':'ADD', 'id':len(changeList), 'nodeName':kwds['name'], 'parentNode':'ROOT', 'isLeaf':'True',  'tagid':0})
        else:
            self.ret_dict['error'] = "Node with this name already exist"        
        self.ret_dict['node_list'] = tree.serialized
        self.ret_dict['token'] = SecurityToken()
        self.ret_dict['changeList'] = pickle.dumps(changeList).replace('\n', 'nL1')
        return self.templatePage("treeEdit", self.ret_dict)

    @expose
    #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    def edit_node(self, *args, **kwds):
        """Applies data about edited node to a change list
        """
        self.ret_dict['error'] = None
        self.ret_dict['currDB'] = kwds['currentDB']
        self.ret_dict['treeName'] = kwds['treeName']
        changeList = pickle.loads(kwds['changeList'].replace('nL1', '\n'))
        tree = Tree(self._getTreeData(kwds['currentDB'], kwds['treeName']), kwds['treeName'])
        tree.applyPreviousChanges(changeList)
        #tree = Tree(kwds['tree'], kwds['treeName'])
        if not tree.hasNode(kwds['new_name']):
            if kwds.has_key('old_name') and kwds.has_key('new_name'):
                ## if kwds['tagId'] != "":
##                     tree.edit_node(kwds['old_name'], kwds['new_name'], kwds['tagId'])
##                     changeList.append({'operation':'CHANGE', 'id':len(changeList), 'nodeName':kwds['old_name'], 'newName':kwds['new_name'], 'tagid':kwds['tagId']})
##                 else:
                tree.edit_node(kwds['old_name'], kwds['new_name'])
                changeList.append({'operation':'CHANGE', 'id':len(changeList), 'nodeName':kwds['old_name'], 'newName':kwds['new_name']})
        else:
            if not(kwds.has_key('old_name') == kwds.has_key('new_name')):
                  self.ret_dict['error'] = "Node with this name already exist" 
        self.ret_dict['node_list'] = tree.serialized
        self.ret_dict['token'] = SecurityToken()
        self.ret_dict['changeList'] = pickle.dumps(changeList).replace('\n', 'nL1')
        return self.templatePage("treeEdit", self.ret_dict)

    @expose
    #@is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))
    @is_authorized(Role("GlobalTag Manager"), Group("CondDB"), onFail=RedirectorToLogin("../GlobalTag/index"))
    def remove_node(self, *args, **kwds):
        """Applies data about node to be removed to a change list
        """
        self.ret_dict['error'] = None
        self.ret_dict['currDB'] = kwds['currentDB']
        self.ret_dict['treeName'] = kwds['treeName']
        changeList = pickle.loads(kwds['changeList'].replace('nL1', '\n'))
        tree = Tree(self._getTreeData(kwds['currentDB'], kwds['treeName']), kwds['treeName'])
        tree.applyPreviousChanges(changeList)
        #tree = Tree(kwds['tree'], kwds['treeName'])
        if kwds.has_key('node'):
            if kwds.has_key('del_children'):
                tree.remove_node(kwds['node'], True, changeList)
            else:
                tree.remove_node(kwds['node'], False, changeList)
        self.ret_dict['node_list'] = tree.serialized
        self.ret_dict['token'] = SecurityToken()
        self.ret_dict['changeList']= pickle.dumps(changeList).replace('\n', 'nL1')
        return self.templatePage("treeEdit", self.ret_dict)

    @expose
    @is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))  
    def viewTagDetails(self, *args, **kwds):
        """Opens a window showing detailed info about selected tag
        """
        self.ret_dict['currDB']= kwds['currentDB']
        tagList = self.data['oracle://'+ kwds['currentDB']][0]
        if kwds.has_key('tagId'):
            for el in tagList:
                if (str(el['tagid'])==kwds['tagId']):
                    self.ret_dict['tag'] = el     
        else:
            for el in tagList:
                if (el['tagname']==kwds['currTag'].split('|')[0].lstrip()):
                    self.ret_dict['tag'] = el
        return self.templatePage("tagInfo", self.ret_dict)

    @expose
    @is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))  
    def viewIOVDetails(self, *args, **kwds):
        """ Redirects user to a window showing IOV information about the selected tag
        """
        self.ret_dict['currDB']= kwds['currentDB']
        tagList = self.data['oracle://'+ kwds['currentDB']][0]
        if kwds.has_key('tagId'):
            for el in tagList:
                if (str(el['tagid'])==kwds['tagId']):
                    self.ret_dict['tag'] = el     
        else:
            for el in tagList:
                if (el['tagname']==kwds['currTag'].split('|')[0].lstrip()):
                    self.ret_dict['tag'] = el
        return self.templatePage("iovInfo", self.ret_dict)

    @expose
    @is_authenticated(onFail=RedirectorToLogin("../GlobalTag/index"))  
    def viewIOVInfo(self):
        self.ret_dict['currDB']= kwds['currentDB']
        self.ret_dict['currTag'] = kwds['currentTag'].split('|')[0].lstrip()
        return self.templatePage("iovInfo", self.ret_dict)

DeclarePlugin ("/Controllers/CondDB/GlobalTag",GlobalTag, {"baseUrl": "/GlobalTag"})


class Tree(object):
    """ Structure allowing modifications of a tree
    one is editing
    """
    def __init__(self, nodeList, treeName):
        self._name = treeName
        self.serialized = self._orderTreeData(nodeList) 
        

    def _orderTreeData(self, dataList):
        """Orders data in a tree fetched from APP SERVER
        so that parent node are before their child nodes
        Important because of GUI(YUI) features
        """
        dataList.sort(lambda x, y: cmp(x['nodelabel'], y['nodelabel']))
        parList = set()
        parList.add('ROOT')
        orderedTree = []
        while (len(dataList) > 0):
            for el in dataList:
                if el['parentlabel'] in parList:
                    orderedTree.append(el)
                    parList.add(el['nodelabel'])
                    dataList.remove(el)
        return orderedTree

    def applyPreviousChanges(self, changeList):
        """Applies changes from a change list, so the previous changes are visible before submit
        """
        for el in changeList:
            if (el['operation']=='ADD'):
                self.insert_node(el['nodeName'], el['parentNode'], el['tagid'])
            elif (el['operation']=='REMOVE'):
                for nds in self.serialized:
                     if nds['nodelabel']==el['nodeName']:
                         for elem in self.serialized:
                             if elem['parentlabel']==el['nodeName']:
                                 elem['parentlabel']=el['parentNode']
                         self.serialized.remove(nds)
            elif (el['operation']=='CHANGE'):
                self.edit_node(el['nodeName'], el['newName'])
            elif (el['operation']=='REMOVE_BRANCH'):
                 for nds in self.serialized:
                    if nds['nodelabel']==el['nodeName']:
                        self._removeChildren(nds['nodelabel'])
                        self.serialized.remove(nds)
    
    def setName(self, name):
        self._name = name
    def name(self):
        return self._name

    def hasNode(self, name):
        for elem in self.serialized:
            if elem['nodelabel']==name:
                return True  
        return False
    
    def insert_node(self,name, parent, tagid):
        for elem in self.serialized:
            if elem['nodelabel']==name:
                return 
        newnd={'nodelabel':name, 'parentlabel': parent, 'tagid':tagid }
        self.serialized.append(newnd)
    
    def edit_node(self, oldName, newName, tagid=None):
        """changes node name, associates tag,
        if changed node has children, changes also their parent name
        """
        for nds in self.serialized:
                if nds['nodelabel']== oldName:
                    nds['nodelabel']=newName
                    if tagid != None:
                        nds['tagid'] = tagid
        for nds in self.serialized:
                if nds['parentlabel']== oldName:
                    nds['parentlabel']=newName
                         
    
    def remove_node(self, node, withChildren, changeList):
        """ removes node from tree
        if removal with children calls a method responsible for removing the whole branch
        if not removes the node and assigns it's children removed node's parent as a parent node
        """
        if (withChildren):
            for nds in self.serialized:
                if nds['nodelabel']==node:
                    self._removeChildren(nds['nodelabel'])
                    self.serialized.remove(nds)
                    changeList.append({'operation':'REMOVE_BRANCH', 'id':len(changeList), 'nodeName':nds['nodelabel']})
        else:
            for nds in self.serialized:
                if nds['nodelabel']==node:
                    for el in self.serialized:
                        if el['parentlabel']==node:
                            el['parentlabel']=nds['parentlabel']
                    self.serialized.remove(nds)
                    changeList.append({'operation':'REMOVE', 'id':len(changeList), 'nodeName':nds['nodelabel']})
                         
    def _removeChildren(self, parentName):
        for nds in self.serialized:
            if nds['parentlabel'] == parentName:
                if nds['tagid']=="0":
                    self._removeChildren(nds['nodelabel'])
                self.serialized.remove(nds)
                self._removeChildren(parentName)
                

                #changeList.append({'operation':'REMOVE', 'id':len(changeList), 'nodeName':nds['nodelabel']})



