from Framework import Controller,StaticController,templatepage
from Framework.PluginManager import DeclarePlugin
from QueryResponder import QueryResponder
from Tools.SecurityModuleCore import SecurityToken, RedirectToLocalPage, RedirectAway
from Tools.SecurityModuleCore import Group, Role, NotAuthenticated, FetchFromArgs
from Tools.SecurityModuleCore import is_authorized, is_authenticated, has_site
from CondDBCommon import RedirectorToLogin

from cherrypy import expose, request  
from cherrypy.lib import static
import os.path
import pickle

class IOVManagementCfg(object):
    templatesPath = staticmethod (lambda : os.path.join(__file__.rsplit ("/", 1)[0], "Templates"))

class IOVManagement(Controller):
    """Page uses iov_tag.tmpl template
    """
    def __init__(self,context):
        self.context=context
        Controller.__init__(self,context,__file__)
        self.__basepath = __file__.rsplit('/', 1)[0]
        self.context.addService (IOVManagementCfg())
        self.__taglist=''
        self.__appurl=context.CondDBCommonCfg().appserverURL()
        #template variable dict
        self.ret_dict={}
        self.ret_dict['servicedict']= {}
        self.ret_dict['chosen_det'] = ''
        self.ret_dict['chosen_service'] = ''
        self.ret_dict['chosen_schema'] = ''
        self.ret_dict['viacache'] = ''
        self.ret_dict['tag_list'] = ''
        self.ret_dict['iov_list'] = ''
        self.ret_dict['file'] = ''
        self.ret_dict['error'] = None
        
    def resetTemplateVariables(self):
        self.ret_dict['chosen_det'] = ''
        self.ret_dict['chosen_service'] = ''
        self.ret_dict['chosen_schema'] = ''
        self.ret_dict['viacache'] = ''
        self.ret_dict['tag_list'] = ''
        self.ret_dict['iov_list'] = ''
        self.ret_dict['file'] = ''
        self.ret_dict['error'] = None
        self.ret_dict['container']=''
        self.ret_dict['destTag']=''
        self.ret_dict['firstSince']=''
        self.ret_dict['lastTill']=''
        if len(self.ret_dict['servicedict'].keys()) == 0:
            #print 'index reloading account list'
            self.ret_dict['servicedict']=self.__buildservicedict()
            self.ret_dict['defaultdet'] = self.ret_dict['servicedict'].keys()[0]
            self.ret_dict['defaultservice'] = self.ret_dict['servicedict'][self.ret_dict['defaultdet']].keys()[0]
            self.ret_dict['defaultschema'] = self.ret_dict['servicedict'][self.ret_dict['defaultdet']][self.ret_dict['defaultservice']][0]

            
    @expose
    @is_authenticated( onFail=RedirectorToLogin("../IOVManagement/index"))
    def index(self):
        """Top level page. Show clean state
        """
        self.resetTemplateVariables()
        return self.templatePage("iov_tag", self.ret_dict)
    @expose    
    def getServiceMap(self):
        """exposed only for debugging purpose
        """
        qResp = QueryResponder(self.__appurl)
        result= qResp.getServiceMap()
        return result
    
    def __buildservicedict(self):
        result={}
        qResp = QueryResponder(self.__appurl)
        r_str=qResp.getServiceMap()
        r=pickle.loads(r_str)
        for (det,value) in r.items():
            result[det]={}
            for v in value:
               [service,schema]=v.split('/')
               if result[det].has_key(service):
                   result[det][service].append(schema)
               else:
                   result[det][service]=[]
                   result[det][service].append(schema)
        return result

    
    @expose
    @is_authenticated( onFail=RedirectorToLogin("../IOVManagement/index"))
    def get_tags(self, *args, **kwds):
        """Fill in the first form to list all available tags for a given service and schema combination        """
        self.resetTemplateVariables()
        self.ret_dict['chosen_det'] = kwds['det']
        self.ret_dict['chosen_service'] = kwds['service']
        self.ret_dict['chosen_schema'] = kwds['schema']
        if kwds.has_key('viacache') and kwds['viacache']!='':
            self.ret_dict['viacache']='CHECKED'
        qResp = QueryResponder(self.__appurl)
        taglist = qResp.getTags(self.ret_dict['chosen_det'],self.ret_dict['chosen_service'],self.ret_dict['chosen_schema'],self.ret_dict['viacache'])
        for tag in taglist:
            self.ret_dict['tag_list']+='<option style="direction:ltr" value="'+tag+'">'+tag+'</option>'
        return self.templatePage("iov_tag", self.ret_dict)

    @expose
    @is_authenticated( onFail=RedirectorToLogin("../IOVManagement/index"))
    def get_iovs(self, *args, **kwds):
        """fill in the second form to display selected tag info and to save result to file. Can be called after get_tags
        """
        self.resetTemplateVariables()
        self.ret_dict['tag_list']=self.__taglist
        if kwds.has_key('viacache') and kwds['viacache']!='':
            self.ret_dict['viacache']='CHECKED'
        if kwds.has_key('sel_tag') is False:
            self.ret_dict['error'] = "Tag has to be selected"
            return self.templatePage("iov_tag", self.ret_dict)
        self.ret_dict['tag_list']=self.__selectTag(kwds['sel_tag'])
        iov_list=self.__prepareIOVList(kwds['det'],kwds['service'],kwds['schema'],kwds['viacache'],kwds['sel_tag'])
        self.ret_dict['iov_list']=iov_list
        if kwds.has_key('file'):           
           qResp = QueryResponder(self.__appurl)
           datafile=qResp.exportIOV(kwds['det'],kwds['service'],kwds['schema'],self.ret_dict['container'],kwds['destTag'],kwds['sel_tag'],kwds['firstSince'],kwds['lastTill'],self.ret_dict['viacache'])
           suggestedfilename='_'.join([self.ret_dict['container'],kwds['sel_tag']])+'.db'
           return static.serve_file(path=datafile,content_type='application/x-download',disposition='attachment',name=suggestedfilename)
        return self.templatePage("iov_tag", self.ret_dict)

    def __selectTag(self,tag):
        """replace the option list with selected tag highlighted
        """
        sel=self.__taglist
        selected = '<option style="direction:ltr" value="'+tag+'">'+tag+'</option>'
        replacement='<option selected="selected" value="'+tag+'">'+tag+'</option>'
        sel.replace(selected,replacement)
        return sel
    
    def __prepareIOVList(self,det,service,schema,viacache,tag):
        """Get info of selected tag from app server
        """
        #list='Tag CSCDBCrosstalk_mc;Time Type 0;PayloadContainerName CSCDBCrosstalkRcd;since \t till \t payloadToken;1 \t  4294967295 \t [DB=7E4ED659-82A4-DA11-A0AC-000E0C5CE043][CNT=HcalGainsRcd][CLID=A069207D-22A2-1BAD-FD51-13613F781A97][TECH=00000B01][OID=00000007-00000000];Total # of payload objects: 1;'
        #new listformat [('[DB=00000000-0000-0000-0000-000000000000][CNT=BeamSpotObjectsRcd][CLID=F0F173D2-520F-D8E4-0202-DE60C35B71FB][TECH=00000B01][OID=00000004-0000000E]', 1L, 4294967295L, 0)]
        
        qResp = QueryResponder(self.__appurl)
        result = pickle.loads(qResp.getIOVForTag(det,service,schema,viacache,tag))
        timetype=result[0]
        fields=result[1]
        totalnumber=len(fields)
        self.ret_dict['totalnumber'] = totalnumber
        self.ret_dict['timetype']=timetype
        #container field
        token=fields[0][0]
        sep='[CNT='
        container = token[token.find(sep)+len(sep):token.find(']',token.find(sep))]
        self.ret_dict['container'] = container
        list =''
        for field in fields:
           list += '<tr>'
           list += '<td class="iov_cell">'+str(field[1])+'</td>'
           list += '<td class="iov_cell">'+str(field[2])+'</td>'
           list+='</tr>'
        return list
    
DeclarePlugin ("/Controllers/CondDB/IOVManagement", IOVManagement, {"baseUrl": "/IOVManagement"})
		  	
